// TestApp1.cpp : Defines the entry point for the application.
//

#ifdef _WIN32
#include "Win32/stdafx.h"
#endif

#include <algorithm>
#include <DK.h>

#define MAX_SIZE 100000000ULL

#ifdef _WIN32
int APIENTRY wWinMain(_In_ HINSTANCE hInstance,
	_In_opt_ HINSTANCE hPrevInstance,
	_In_ LPWSTR    lpCmdLine,
	_In_ int       nCmdShow)
#else
int main(int argc, const char * argv[])
#endif
{
	union Value
	{
		size_t i64;
		struct
		{
			uint32_t h, l;
		} i32;
	};



	auto testStd = [](Value* val_copy, const Value* input, size_t len)->DKHashResultSHA1
	{
		DKLogI("Copying buffer...");
		memcpy(val_copy, input, sizeof(Value) * len);

		DKTimer timer;
		DKLogI("Sorting by std::sort...");
		timer.Reset();
		std::sort(val_copy, val_copy + MAX_SIZE, [](const Value& lhs, const Value& rhs)->bool
		{
			return lhs.i64 < rhs.i64;
		});
		double e1 = timer.Elapsed();

		DKLogI("std::sort: %f", e1);
		return DKHashSHA1(val_copy, sizeof(Value)*len);
	};
	auto testDK = [](Value* val_copy, const Value* input, size_t len)->DKHashResultSHA1
	{
		DKLogI("Copying buffer...");
		memcpy(val_copy, input, sizeof(Value) * len);

		DKStaticArray<Value> tmp(val_copy, MAX_SIZE);

		DKTimer timer;
		DKLogI("Sorting by DKStaticArray::Sort...");
		timer.Reset();
		tmp.Sort([](const Value& lhs, const Value& rhs)->bool
		{
			return lhs.i64 < rhs.i64;
		});
		double e1 = timer.Elapsed();

		DKLogI("DKStaticArray::sort: %f (memCopy:%d)", e1, tmp.UseMemoryCopy);
		return DKHashSHA1(val_copy, sizeof(Value)*len);
	};

	DKApplication app;
	DKLog("Allocating buffer...");

	Value* val1 = new Value[MAX_SIZE];

	for (size_t i = 0; i < MAX_SIZE; ++i)
	{
		val1[i].i32 = { DKRandom(), DKRandom() };
	}
	Value* val_copy = new Value[MAX_SIZE];

	DKHashResultSHA1 sha1_r1 = testDK(val_copy, val1, MAX_SIZE);
	DKLogI("Hash1: %ls", (const wchar_t*)sha1_r1.String());
	DKHashResultSHA1 sha1_r2 = testStd(val_copy, val1, MAX_SIZE);
	DKLogI("Hash2: %ls", (const wchar_t*)sha1_r2.String());
	DKLogI("Hash compare:%d", sha1_r1.Compare(sha1_r2));

	delete[] val1;
	delete[] val_copy;

	return 0;
}
